==> botocss-core-4.3-sources.jar: com/atlassian/botocss/Botocss.java <==
package com.atlassian.botocss;

import java.util.ArrayList;
import java.util.List;
import java.util.regex.Pattern;

import com.google.common.base.Function;

import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.select.Selector;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import cz.vutbr.web.css.CombinedSelector;
import cz.vutbr.web.css.Declaration;
import cz.vutbr.web.css.RuleBlock;
import cz.vutbr.web.css.RuleSet;
import cz.vutbr.web.css.StyleSheet;
import cz.vutbr.web.css.Term;
import cz.vutbr.web.css.TermAngle;
import cz.vutbr.web.css.TermColor;
import cz.vutbr.web.css.TermIdent;
import cz.vutbr.web.css.TermLength;
import cz.vutbr.web.css.TermNumber;
import cz.vutbr.web.css.TermNumeric;
import cz.vutbr.web.css.TermPercent;
import cz.vutbr.web.css.TermString;
import cz.vutbr.web.css.TermTime;
import cz.vutbr.web.css.TermURI;

import static java.util.Collections.emptyList;

/**
 * Public facade for Botocss. Use {@link #inject(String, String...)} as a simple method
 * of injecting CSS stylesheets into an HTML document.
 */
public final class Botocss
{
    private static final Logger log = LoggerFactory.getLogger(Botocss.class);

    private static final Pattern CAN_SHORTEN_COLOR_PATTERN = Pattern.compile("^#([0-9a-f])\\1([0-9a-f])\\2([0-9a-f])\\3$");
    private static final String SHORTEN_COLOR_REPLACEMENT = "#$1$2$3";

    /**
     * Injects the CSS from the stylesheets into the provided HTML as inline styles.
     * Also applies any inline &lt;style&gt; elements found in the HTML content.
     * <p/>
     * See the Botocss home page for documentation and caveats around the
     * injection process.
     * 
     * @param html the HTML document
     * @param stylesheets the stylesheets containing the CSS to inject
     * @throws NullPointerException if the provided HTML or any of the stylesheets are {@code null}
     * @return the injected HTML string
     */
    public static String inject(String html, String... stylesheets) {
        log.debug("Parsing external stylesheets");
        BotocssStyles styles = BotocssStyles.parse(stylesheets);
        return inject(html, styles);
    }

    
    /**
     * Injects parsed stylesheets into the provided HTML as inline styles.
     * Also applies any inline &lt;style&gt; elements found in the HTML content.
     * Use {@link #parse} to create a stylesheet object.
     * <p/>
     * See the Botocss home page for documentation and caveats around the
     * injection process.
     * 
     * @param html the HTML document
     * @param styles the pre-parsed stylesheets containing the CSS to inject
     * @return the injected HTML string
     * @throws NullPointerException if the provided HTML or styles are {@code null}
     * @see #parse
     */
    public static String inject(String html, BotocssStyles styles) {
        return inject(html, styles, DocumentFunctions.PRETTY_PRINT);
    }
    
    /**
     * Injects parsed stylesheets into the provided HTML as inline styles.
     * Also applies any inline &lt;style&gt; elements found in the HTML content.
     * Use {@link #parse} to create a stylesheet object.
     * <p/>
     * See the Botocss home page for documentation and caveats around the
     * injection process.
     * 
     * @param html the HTML document
     * @param styles the pre-parsed stylesheets containing the CSS to inject
     * @param documentProcessor modify the parsed HTML document after injection before serializing
     * @return the injected HTML string
     * @throws NullPointerException if the provided HTML or styles are {@code null}
     * @see #parse
     */
    public static String inject(String html, BotocssStyles styles, Function<Document, Document> documentFunction) {
	long start = System.currentTimeMillis();

        Document documentWithCSSInlined = Jsoup.parse(html);
        log.debug("Parsed HTML document in {} ms", System.currentTimeMillis() - start);

        int selectorCount = 0;
        
        log.debug("Applying external stylesheets");
        selectorCount += applyStyles(documentWithCSSInlined, styles);

        log.debug("Finding inline stylesheets");
        List<String> inlineCss = new ArrayList<String>();
        for (Element inlineStyle : documentWithCSSInlined.getElementsByTag("style")) {
            inlineCss.add(inlineStyle.html());
        }
        if (!inlineCss.isEmpty()) {
            log.debug("Parsing inline stylesheets");
            BotocssStyles inlineStyles = BotocssStyles.parse(inlineCss.toArray(new String[inlineCss.size()]));

            log.debug("Applying inline stylesheets");
            selectorCount += applyStyles(documentWithCSSInlined, inlineStyles);
        }

        Document processedDocumentWithCSSInlined = documentFunction.apply(documentWithCSSInlined);
        if(processedDocumentWithCSSInlined == null) {
            processedDocumentWithCSSInlined = documentWithCSSInlined; // default to the given document
        }
        
        String result = processedDocumentWithCSSInlined.outerHtml();

        log.info("Applying {} CSS selectors to HTML (length {}) took {} ms",
                new Object[] { selectorCount, html.length(), System.currentTimeMillis() - start });
        return result;
    }

    /**
     * Parses the provided CSS and returns a list of pre-parsed Stylesheet objects for
     * use with {@link #inject(String, BotocssStyles)}.
     *
     * @param stylesheets the CSS stylesheets to parse
     * @throws NullPointerException if any of the provided stylesheets are {@code null}
     * @see #inject(String, BotocssStyles)
     */
    public static BotocssStyles parse(String... stylesheets) {
        return BotocssStyles.parse(stylesheets);
    }
    
    private static int applyStyles(Document document, BotocssStyles styles) {
        int selectorCount = 0;
        for (StyleSheet styleSheet : styles.getStyleSheets()) {
            selectorCount += applyStylesheet(document, styleSheet);
        }
        return selectorCount;
    }

    private static int applyStylesheet(Document document, StyleSheet stylesheet) {
        int selectorCount = 0;

        for (RuleBlock<?> block : stylesheet) {
            if (!(block instanceof RuleSet)) continue;

            RuleSet set = (RuleSet) block;
            for (CombinedSelector selector : set.getSelectors()) {
                selectorCount++;
                log.debug("Applying selector #{}: {}", selectorCount, selector.toString());
                List<Element> elements = findElements(document, selector);
                for (Element element : elements) {
                    for (Declaration declaration : set) {
                        log.debug("Applying style [ {} ] to element: {}", declaration.toString().trim(),
                            element.nodeName());
                        try {
                            String rule = declaration.getProperty() + ": " + getStringValue(declaration);
                            String existingStyle = element.attr("style");
                            element.attr("style", existingStyle.equals("") ? rule : existingStyle + "; " + rule);
                        } catch (IllegalArgumentException e) {
                            log.warn("Failed to process CSS property value: " + e.getMessage());
                        }
                    }
                }
            }
        }

        return selectorCount;
    }

    private static List<Element> findElements(Document document, CombinedSelector selector) {
        try {
            return document.select(selector.toString());
        } catch (Selector.SelectorParseException e) {
            log.info("Skipping unsupported selector: " + selector.toString());
            return emptyList();
        }
    }


    private static String getStringValue(Declaration declaration) {
        StringBuilder result = new StringBuilder();
        for (int i = 0; i < declaration.size(); i++) {
            Term<?> term = declaration.get(i);
            if (i > 0) result.append(term.getOperator().value());
            result.append(getStringValue(term));
        }
        return result.toString();
    }

    private static String getStringValue(Term<?> term) {
        if (term instanceof TermIdent) {
            return (String) term.getValue();
        }
        if (term instanceof TermString) {
            return "'" + ((String) term.getValue()).replaceAll("'", "\\\\'") + "'";
        }
        if (term instanceof TermNumber) {
            return formatNumber(((TermNumber) term).getValue());
        }
        if (term instanceof TermLength || term instanceof TermAngle || term instanceof TermTime) {
            TermNumeric<Float> length = (TermNumeric<Float>) term;
            return formatNumber(length.getValue()) + length.getUnit().value();
        }
        if (term instanceof TermPercent) {
            return formatNumber(((TermPercent) term).getValue()) + "%";
        }
        if (term instanceof TermColor) {
            java.awt.Color color = ((TermColor) term).getValue();
            String hex = String.format("#%02x%02x%02x", color.getRed(), color.getGreen(), color.getBlue());
            return CAN_SHORTEN_COLOR_PATTERN.matcher(hex).replaceAll(SHORTEN_COLOR_REPLACEMENT);
        }
        if (term instanceof TermURI) {
            return "url(" + term.getValue() + ")";
        }
        throw new IllegalArgumentException("Unrecognised CSS value: " + term.toString() +
            " (" + term.getClass() + ")");
    }

    private static String formatNumber(float value) {
        if (value == Math.ceil(value))
            return String.valueOf(Math.round(value));
        else
            return String.format("%.3f", value);
    }
}

==> botocss-core-5.2-sources.jar: com/atlassian/botocss/Botocss.java <==
package com.atlassian.botocss;

import java.util.ArrayList;
import java.util.List;
import java.util.regex.Pattern;

import com.google.common.base.Function;

import com.google.common.base.Joiner;
import cz.vutbr.web.css.*;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.select.Selector;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import static java.util.Collections.emptyList;

/**
 * Public facade for Botocss. Use {@link #inject(String, String...)} as a simple method
 * of injecting CSS stylesheets into an HTML document.
 */
public final class Botocss
{
    private static final Logger log = LoggerFactory.getLogger(Botocss.class);

    private static final Pattern CAN_SHORTEN_COLOR_PATTERN = Pattern.compile("^#([0-9a-f])\\1([0-9a-f])\\2([0-9a-f])\\3$");
    private static final String SHORTEN_COLOR_REPLACEMENT = "#$1$2$3";

    /**
     * Injects the CSS from the stylesheets into the provided HTML as inline styles.
     * Also applies any inline &lt;style&gt; elements found in the HTML content.
     * <p/>
     * See the Botocss home page for documentation and caveats around the
     * injection process.
     * 
     * @param html the HTML document
     * @param stylesheets the stylesheets containing the CSS to inject
     * @throws NullPointerException if the provided HTML or any of the stylesheets are {@code null}
     * @return the injected HTML string
     */
    public static String inject(String html, String... stylesheets) {
        log.debug("Parsing external stylesheets");
        BotocssStyles styles = BotocssStyles.parse(stylesheets);
        return inject(html, styles);
    }

    
    /**
     * Injects parsed stylesheets into the provided HTML as inline styles.
     * Also applies any inline &lt;style&gt; elements found in the HTML content.
     * Use {@link #parse} to create a stylesheet object.
     * <p/>
     * See the Botocss home page for documentation and caveats around the
     * injection process.
     * 
     * @param html the HTML document
     * @param styles the pre-parsed stylesheets containing the CSS to inject
     * @return the injected HTML string
     * @throws NullPointerException if the provided HTML or styles are {@code null}
     * @see #parse
     */
    public static String inject(String html, BotocssStyles styles) {
        return inject(html, styles, DocumentFunctions.PRETTY_PRINT);
    }
    
    /**
     * Injects parsed stylesheets into the provided HTML as inline styles.
     * Also applies any inline &lt;style&gt; elements found in the HTML content.
     * Use {@link #parse} to create a stylesheet object.
     * <p/>
     * See the Botocss home page for documentation and caveats around the
     * injection process.
     * 
     * @param html the HTML document
     * @param styles the pre-parsed stylesheets containing the CSS to inject
     * @param documentProcessor modify the parsed HTML document after injection before serializing
     * @return the injected HTML string
     * @throws NullPointerException if the provided HTML or styles are {@code null}
     * @see #parse
     */
    public static String inject(String html, BotocssStyles styles, Function<Document, Document> documentFunction) {
	long start = System.currentTimeMillis();

        Document documentWithCSSInlined = Jsoup.parse(html);
        log.debug("Parsed HTML document in {} ms", System.currentTimeMillis() - start);

        int selectorCount = 0;
        
        log.debug("Applying external stylesheets");
        selectorCount += applyStyles(documentWithCSSInlined, styles);

        log.debug("Finding inline stylesheets");
        List<String> inlineCss = new ArrayList<String>();
        for (Element inlineStyle : documentWithCSSInlined.getElementsByTag("style")) {
            inlineCss.add(inlineStyle.html());
        }
        if (!inlineCss.isEmpty()) {
            log.debug("Parsing inline stylesheets");
            BotocssStyles inlineStyles = BotocssStyles.parse(inlineCss.toArray(new String[inlineCss.size()]));

            log.debug("Applying inline stylesheets");
            selectorCount += applyStyles(documentWithCSSInlined, inlineStyles);
        }

        Document processedDocumentWithCSSInlined = documentFunction.apply(documentWithCSSInlined);
        if(processedDocumentWithCSSInlined == null) {
            processedDocumentWithCSSInlined = documentWithCSSInlined; // default to the given document
        }
        
        String result = processedDocumentWithCSSInlined.outerHtml();

        log.info("Applying {} CSS selectors to HTML (length {}) took {} ms",
                new Object[] { selectorCount, html.length(), System.currentTimeMillis() - start });
        return result;
    }

    /**
     * Parses the provided CSS and returns a list of pre-parsed Stylesheet objects for
     * use with {@link #inject(String, BotocssStyles)}.
     *
     * @param stylesheets the CSS stylesheets to parse
     * @throws NullPointerException if any of the provided stylesheets are {@code null}
     * @see #inject(String, BotocssStyles)
     */
    public static BotocssStyles parse(String... stylesheets) {
        return BotocssStyles.parse(stylesheets);
    }
    
    private static int applyStyles(Document document, BotocssStyles styles) {
        int selectorCount = 0;
        for (StyleSheet styleSheet : styles.getStyleSheets()) {
            selectorCount += applyStylesheet(document, styleSheet);
        }
        return selectorCount;
    }

    private static int applyStylesheet(Document document, StyleSheet stylesheet) {
        int selectorCount = 0;

        for (RuleBlock<?> block : stylesheet) {
            if (!(block instanceof RuleSet)) continue;

            RuleSet set = (RuleSet) block;
            for (CombinedSelector selector : set.getSelectors()) {
                selectorCount++;
                log.debug("Applying selector #{}: {}", selectorCount, selector.toString());
                List<Element> elements = findElements(document, selector);
                for (Element element : elements) {
                    for (Declaration declaration : set) {
                        log.debug("Applying style [ {} ] to element: {}", declaration.toString().trim(),
                            element.nodeName());
                        try {
                            String rule = declaration.getProperty() + ": " + getStringValue(declaration);
                            String existingStyle = element.attr("style");
                            element.attr("style", existingStyle.equals("") ? rule : existingStyle + "; " + rule);
                        } catch (IllegalArgumentException e) {
                            log.warn("Failed to process CSS property value: " + e.getMessage());
                        }
                    }
                }
            }
        }

        return selectorCount;
    }

    private static List<Element> findElements(Document document, CombinedSelector selector) {
        try {
            return document.select(selector.toString());
        } catch (Selector.SelectorParseException e) {
            log.info("Skipping unsupported selector: " + selector.toString());
            return emptyList();
        }
    }


    private static String getStringValue(Declaration declaration) {
        StringBuilder result = new StringBuilder();
        for (int i = 0; i < declaration.size(); i++) {
            Term<?> term = declaration.get(i);
            if (i > 0) result.append(term.getOperator().value());
            result.append(getStringValue(term));
        }
        return result.toString();
    }

    private static String getStringValue(Term<?> term) {
        if (term instanceof TermIdent) {
            return (String) term.getValue();
        }
        if (term instanceof TermString) {
            return "'" + ((String) term.getValue()).replaceAll("'", "\\\\'") + "'";
        }
        if (term instanceof TermNumber) {
            return formatNumber(((TermNumber) term).getValue());
        }
        if (term instanceof TermLength || term instanceof TermAngle || term instanceof TermTime) {
            TermNumeric<Float> length = (TermNumeric<Float>) term;
            return formatNumber(length.getValue()) + length.getUnit().value();
        }
        if (term instanceof TermPercent) {
            return formatNumber(((TermPercent) term).getValue()) + "%";
        }
        if (term instanceof TermFunction) {
            TermFunction function = (TermFunction) term;
            return function.getFunctionName() + "(" + Joiner.on("").join(function) + ")";
        }
        if (term instanceof TermColor) {
            java.awt.Color color = ((TermColor) term).getValue();
            String hex = String.format("#%02x%02x%02x", color.getRed(), color.getGreen(), color.getBlue());
            return CAN_SHORTEN_COLOR_PATTERN.matcher(hex).replaceAll(SHORTEN_COLOR_REPLACEMENT);
        }
        if (term instanceof TermURI) {
            return "url(" + term.getValue() + ")";
        }
        throw new IllegalArgumentException("Unrecognised CSS value: " + term.toString() +
            " (" + term.getClass() + ")");
    }

    private static String formatNumber(float value) {
        if (value == Math.ceil(value))
            return String.valueOf(Math.round(value));
        else
            return String.format("%.3f", value);
    }
}
